@@ -1,5 +1,5 @@
 /*
- * Copyright 2007-2008 by Sascha Hlusiak. <saschahlusiak@freedesktop.org>     
+ * Copyright 2007-2009 by Sascha Hlusiak. <saschahlusiak@freedesktop.org>     
  * Copyright 1995-1999 by Frederic Lepied, France. <Lepied@XFree86.org>       
  *                                                                            
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -44,6 +44,9 @@
 #include "jstk_key.h"
 #include "jstk_options.h"
 #include "jstk_properties.h"
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 3
+    #include <xserver-properties.h>
+#endif
 
 #ifdef LINUX_BACKEND
     #include "backend_joystick.h"
@@ -283,20 +286,9 @@ jstkReadProc(LocalDevicePtr local)
 
             case JSTK_MAPPING_KEY: if (priv->keys_enabled == TRUE) {
                 if (priv->axis[number].type == JSTK_TYPE_ACCELERATED) {
-                    if ((priv->axis[number].value > 0) != 
-                        (priv->axis[number].oldvalue > 0))
-                        jstkGenerateKeys(priv->keyboard_device, 
-                                         priv->axis[number].keys_high,
-                                         (priv->axis[number].value > 0) ? 1:0);
-
-                    if ((priv->axis[number].value < 0) != 
-                        (priv->axis[number].oldvalue < 0))
-                        jstkGenerateKeys(priv->keyboard_device,
-                                         priv->axis[number].keys_low,
-                                         (priv->axis[number].value < 0) ? 1:0);
+                    jstkHandlePWMAxis(local, number);
                 } else if (priv->axis[number].type == JSTK_TYPE_BYVALUE) {
-                    if (priv->keys_enabled == TRUE)
-                        jstkStartAxisTimer(local, number);
+                    jstkStartAxisTimer(local, number);
                 }
                 break;
             }
@@ -337,6 +329,10 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
     int i;
     LocalDevicePtr   local = (LocalDevicePtr)pJstk->public.devicePrivate;
     JoystickDevPtr   priv  = (JoystickDevPtr)XI_PRIVATE(pJstk);
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+    Atom             btn_labels[BUTTONMAP_SIZE+1] = {0}; /* TODO: fillme */
+    Atom             axes_labels[MAXAXES] = {0}; /* TODO: fillme */
+#endif
 
     switch (what) {
     case DEVICE_INIT: {
@@ -352,9 +348,16 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
             priv->close_proc(priv);
         }
 
-        for (m=0; m<=BUTTONMAP_SIZE; m++)
+        for (m=0; m<=BUTTONMAP_SIZE; m++) {
             buttonmap[m] = m;
+	}
+
+            
+            
         if (InitButtonClassDeviceStruct(pJstk, BUTTONMAP_SIZE, 
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+            btn_labels,
+#endif
             buttonmap) == FALSE) {
             ErrorF("unable to allocate Button class device\n");
             return !Success;
@@ -372,6 +375,9 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
 
         if (InitValuatorClassDeviceStruct(pJstk, 
                                           m,
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+                                          axes_labels,
+#endif
 #if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 3
                                           xf86GetMotionEvents, 
 #endif
@@ -382,6 +388,9 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
         } else {
             InitValuatorAxisStruct(pJstk,
                                    0, /* valuator num */
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+                                   XIGetKnownProperty(AXIS_LABEL_PROP_REL_X),
+#endif
                                    0, /* min val */
                                    screenInfo.screens[0]->width, /* max val */
                                    1, /* resolution */
@@ -389,6 +398,9 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
                                    1); /* max_res */
             InitValuatorAxisStruct(pJstk,
                                    1, /* valuator num */
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+                                   XIGetKnownProperty(AXIS_LABEL_PROP_REL_Y),
+#endif
                                    0, /* min val */
                                    screenInfo.screens[0]->height, /* max val */
                                    1, /* resolution */
@@ -399,6 +411,9 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
             {
                 InitValuatorAxisStruct(pJstk,
                                        priv->axis[i].valuator,
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+                                       axes_labels[i],
+#endif
                                        -32768, /* min val */
                                        32767,  /* max val */
                                        1, /* resolution */
@@ -449,6 +464,12 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
             priv->timerrunning = FALSE;
             TimerCancel(priv->timer);
         }
+        for (i = 0; i < MAXAXES; i++)
+            if (priv->axis[i].timerrunning)
+        {
+            priv->axis[i].timerrunning = FALSE;
+            TimerCancel(priv->axis[i].timer);
+        }
 
         if (local->fd >= 0)
             RemoveEnabledDevice(local->fd);
@@ -519,7 +540,7 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     local->private = priv;
     local->type_name = XI_JOYSTICK;
     local->history_size = 0;
-    local->always_core_feedback = 0;
+    local->always_core_feedback = NULL;
     local->conf_idev = dev;
 
     priv->fd = -1;
@@ -534,8 +555,6 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     priv->keys_enabled = TRUE;
     priv->amplify = 1.0f;
     priv->keyboard_device = NULL;
-    priv->keymap.size = 1;
-    memset(priv->keymap.map, NoSymbol, sizeof(priv->keymap.map));
     priv->repeat_delay = 0;
     priv->repeat_interval = 0;
     priv->num_axes    = MAXAXES;
@@ -552,6 +571,9 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
         priv->axis[i].amplify      = 1.0f;
         priv->axis[i].valuator     = -1;
         priv->axis[i].subpixel     = 0.0f;
+        priv->axis[i].timer        = NULL;
+        priv->axis[i].timerrunning = FALSE;
+        priv->axis[i].key_isdown   = 0;
         for (j=0; j<MAXKEYSPERBUTTON; j++)
             priv->axis[i].keys_low[j] = priv->axis[i].keys_high[j] = 0;
     }
@@ -561,6 +583,7 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
         priv->button[i].mapping      = JSTK_MAPPING_NONE;
         priv->button[i].currentspeed = 1.0f;
         priv->button[i].subpixel     = 0.0f;
+        priv->button[i].amplify      = 1.0;
         for (j=0; j<MAXKEYSPERBUTTON; j++)
             priv->button[i].keys[j] = 0;
     }
@@ -635,6 +658,26 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
         }
         xfree(s);
     }
+    
+    priv->rmlvo.rules = xf86SetStrOption(dev->commonOptions, "xkb_rules", NULL);
+    if (!priv->rmlvo.rules)
+	priv->rmlvo.rules = xf86SetStrOption(dev->commonOptions, "XkbRules", "evdev");
+
+    priv->rmlvo.model = xf86SetStrOption(dev->commonOptions, "xkb_model", NULL);
+    if (!priv->rmlvo.model)
+	priv->rmlvo.model = xf86SetStrOption(dev->commonOptions, "XkbModel", "evdev");
+
+    priv->rmlvo.layout = xf86SetStrOption(dev->commonOptions, "xkb_layout", NULL);
+    if (!priv->rmlvo.layout)
+	priv->rmlvo.layout = xf86SetStrOption(dev->commonOptions, "XkbLayout", "us");
+
+    priv->rmlvo.variant = xf86SetStrOption(dev->commonOptions, "xkb_variant", NULL);
+    if (!priv->rmlvo.variant)
+	priv->rmlvo.variant = xf86SetStrOption(dev->commonOptions, "XkbVariant", "");
+
+    priv->rmlvo.options = xf86SetStrOption(dev->commonOptions, "xkb_options", NULL);
+    if (!priv->rmlvo.options)
+	priv->rmlvo.options = xf86SetStrOption(dev->commonOptions, "XkbOptions", "");
 
     priv->mouse_enabled = xf86SetBoolOption(dev->commonOptions, "StartMouseEnabled", TRUE);
     priv->keys_enabled = xf86SetBoolOption(dev->commonOptions, "StartKeysEnabled", TRUE);
